@@ -29,10 +29,12 @@
  */
 
 
-#define MODULE_VERSION  "1.11"
+#define MODULE_VERSION  "1.13"
+
+#define PY_SSIZE_T_CLEAN
 
 #include <Python.h>
-#include <lzo1x.h>
+#include <lzo/lzo1x.h>
 
 /* Python 2x3 compatible macros */
 #if PY_VERSION_HEX >= 0x03000000
@@ -83,7 +85,7 @@ compress(PyObject *dummy, PyObject *args)
     lzo_uint in_len;
     lzo_uint out_len;
     lzo_uint new_len;
-    int len;
+    Py_ssize_t len;
     int level = 1;
     int header = 1;
     int err;
@@ -95,6 +97,16 @@ compress(PyObject *dummy, PyObject *args)
     if (len < 0)
         return NULL;
 
+    if (len > LZO_UINT_MAX) {
+      PyErr_SetString(LzoError, "Input size is larger than LZO_UINT_MAX");
+      return NULL;
+    }
+
+    if ((len + len / 16 + 64 + 3) > LZO_UINT_MAX) {
+      PyErr_SetString(LzoError, "Output size is larger than LZO_UINT_MAX");
+      return NULL;
+    }
+
     in_len = len;
     out_len = in_len + in_len / 16 + 64 + 3;
 
@@ -123,6 +135,7 @@ compress(PyObject *dummy, PyObject *args)
     out = (lzo_bytep) PyString_AsString(result_str);
 #endif
 
+    Py_BEGIN_ALLOW_THREADS
     outc = header ? out+5 : out; // leave space for header if needed
     new_len = out_len;
     if (level == 1)
@@ -137,6 +150,8 @@ compress(PyObject *dummy, PyObject *args)
             out[0] = 0xf1;
         err = lzo1x_999_compress(in, in_len, outc, &new_len, wrkmem);
     }
+    Py_END_ALLOW_THREADS
+
     PyMem_Free(wrkmem);
     if (err != LZO_E_OK || new_len > out_len)
     {
@@ -186,7 +201,7 @@ decompress(PyObject *dummy, PyObject *args)
     lzo_uint in_len;
     lzo_uint out_len;
     lzo_uint new_len;
-    int len;
+    Py_ssize_t len;
     int buflen = -1;
     int header = 1;
     int err;
@@ -225,8 +240,12 @@ decompress(PyObject *dummy, PyObject *args)
 #else
     out = (lzo_bytep) PyString_AsString(result_str);
 #endif
+
+    Py_BEGIN_ALLOW_THREADS
     new_len = out_len;
     err = lzo1x_decompress_safe(in, in_len, out, &new_len, NULL);
+    Py_END_ALLOW_THREADS
+
     if (err != LZO_E_OK || (header && new_len != out_len) )
     {
         Py_DECREF(result_str);
@@ -267,7 +286,7 @@ optimize(PyObject *dummy, PyObject *args)
     lzo_uint in_len;
     lzo_uint out_len;
     lzo_uint new_len;
-    int len;
+    Py_ssize_t len;
     int err;
     int header = 1;
     int buflen = -1;
@@ -311,8 +330,12 @@ optimize(PyObject *dummy, PyObject *args)
 #else
     in = (lzo_bytep) PyString_AsString(result_str);
 #endif
+
+    Py_BEGIN_ALLOW_THREADS
     new_len = out_len;
     err = lzo1x_optimize( header ? in+5 : in, in_len, out, &new_len, NULL);
+    Py_END_ALLOW_THREADS
+
     PyMem_Free(out);
     if (err != LZO_E_OK || (header && new_len != out_len))
     {
@@ -345,14 +368,19 @@ static PyObject *
 adler32(PyObject *dummy, PyObject *args)
 {
     char *buf;
-    int len;
+    Py_ssize_t len;
     unsigned long val = 1; /* == lzo_adler32(0, NULL, 0); */
 
     UNUSED(dummy);
     if (!PyArg_ParseTuple(args, "s#|l", &buf, &len, &val))
         return NULL;
     if (len > 0)
+    {
+        Py_BEGIN_ALLOW_THREADS
         val = lzo_adler32((lzo_uint32)val, (const lzo_bytep)buf, len);
+        Py_END_ALLOW_THREADS
+    }
+
 #if PY_MAJOR_VERSION >= 3
     return PyLong_FromLong(val);
 #else
@@ -376,7 +404,7 @@ static PyObject *
 crc32(PyObject *dummy, PyObject *args)
 {
     char *buf;
-    int len;
+    Py_ssize_t len;
     unsigned long val = 0; /* == lzo_crc32(0, NULL, 0); */
 
     UNUSED(dummy);
